<?php 
/**
 * $Horde$
 *
 * Horde_links class
 *
 * Copyright 2003, Jeroen Huinink <j.huinink@wanadoo.nl>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 * 
 * @author  Jeroen Huinink <j.huinink@wanadoo.nl>
 * @version $Revision$
 * @since   Horde 3.0
 * @package horde.links
 *
 * We refer to link_type as we mean the (string) link types name or identifier. 
 * We refer to link_definition if we mean the (hash) with link properties.
 * In the _link_definitions hash the link_definitions are identified by 
 * the link_types.
 */

class Horde_Links {
  
    //TODO: I borrowed from various other Horde classes, e.g. Prefs, Auth
    // and I didn't update all the comments yet, so you might find reference
    // to other classes in the comments or superfluous methods or properties.

    /**
     * Name of the application from which the links are created.
     *
     * @access private
     *
     * @var string $_from_application
     */
    var $_from_application;

    /**
     * Hash with available link definitions
     *
     * @access private
     *
     * @var string $_link_definitions;
     *
     */
    var $_link_definitions = array();

    /*
     * Array to hold retrieved links
     *
     * @access private
     *
     * @var string $_links;
     *
     */
    var $_links = array();

    /**
     * Attempts to return a concrete Horde_Links instance based on 
     * $driver.
     *
     * @param string $from_appliction    A string identifying the source 
     *                                   application ('provides')
     *
     * @param optional array $params     A hash containing any additional
     *                                   configuration or connection
     *                                   parameters a subclass might need.
     *
     * @access public
     *
     * @return object Horde_Links        The newly created concrete 
     *                                   Horde_Links instance, or false
     *                                   on an error.
     */
    function &factory($driver, $from_application, $params = null)
    {
        if(is_array($driver)) {
            list($app, $driver) = $driver;
        }

        $driver= basename($driver);

        if (empty($driver) || strcmp($driver, 'none') == 0) {
            return new Horde_Links();
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('links',$driver);
        }

        if (!empty($app)) {
            include_once $GLOBALS['registry']->getParam('fileroot', $app) . '/lib/Links/' . $driver . '.php';
        } elseif (@file_exists(dirname(__FILE__) . '/Links/' . $driver . '.php')) {
            include_once dirname(__FILE__) . '/Links/' . $driver . '.php';
        } else {
            @include_once 'Horde/Links/' . $driver . '.php';
        }
        $class = 'Horde_Links_' . $driver;
        if (class_exists($class)) {
            return new $class($from_application, $params);
        } else {
            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
        }
    }

    /**
     * Attempts to return a reference to a concrete Horde_Links 
     * instance based on $driver. It will only create a new instance if no
     * Horde_Links instance with the same parameter currently 
     * exists.
     *
     * This method must be invoked as: $var = &Horde_Links::singleton()
     *
     * @param string $from_application     The 'provides' value for the 
     *                                     application requesting the links 
     *                                     or an application name.
     *
     * @param optional mixed $driver       If no driver is specified we read
     *                                     the driver from the configuration.
     * TODO: is this something that we should do within the framework? I 
     * haven't seen other optional $driver parameters
     *
     * @param optional array $params     A hash containing any additional
     *                                   configuration or connection
     *                                   parameters a subclass might need.
     *
     * @return object Horde_Links  The concrete Horde_Links 
     *                                     reference or false on an error.
     * 
     * @access public
     */
    function &singleton($from_application, $driver = '', $params = null)
    {
        global $conf, $registry;

        static $instances;
        if (empty($driver)) {
            $driver = $conf['links']['driver'];
        }

        if ($registry->getParam('provides', $from_application) != '') {
            $from_application = $registry->getParam('provides', $from_application);
        }
        if (!isset($instances)) {
            $instances = array();
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('links', $driver);
        }

        $signature = serialize(array($driver, $from_application, $params));
        if (!array_key_exists($signature, $instances)) {
            $instances[$signature] = &Horde_Links::factory($driver, $from_application, $params);
        }

        return $instances[$signature];
    }

    /** 
     * Constructor
     *
     * @param string $from_application     The 'provides' value for the 
     *                                     application
     *
     * @access public
     */
    function Horde_Links($from_application = false)
    {
        global $registry;
        if ($from_application) {
            $this->_from_application = $from_application;
            $link_definitions = $registry->getParam('links');
            if (isset($link_definitions) && count($link_definitions)) {
                foreach ($link_definitions as $link_type => $link_definition) {
                    $this->_link_definitions[$this->_from_application . '/' . $link_type]['provider'] = $link_definition['provider'];
                    $this->_link_definitions[$this->_from_application . '/' . $link_type]['description'] = $link_definition['description'];
                    $this->_link_definitions[$this->_from_application . '/' . $link_type]['type'] = $link_type;
                    $this->_link_definitions[$this->_from_application . '/' . $link_type]['readonly'] = (isset($link_definition['readonly']) && $link_definition['readonly']);
                }
            } else {
                $this->_link_definitions= array();
            }

            $provides = $registry->getParam('provides');
            foreach($registry->listApps() as $app) {
                $link_definitions= $registry->getParam('links', $app);
                if (isset($link_definitions) && (count($link_definitions) > 0)) {
                    foreach($link_definitions as $link_type => $link_definition) {
                        if ($link_definition['provider'] == $provides) {
                            if (!empty($link_definition['reverse'])) {
                                $reverse_provider = $registry->getParam('provides', $app);
                                if (isset($link_definition['reverse']['description'])) {
                                    $reverse_description = $link_definition['reverse']['description'];
                                } else {
                                    $reverse_description = $link_definition['description'];
                                }
                                $this->_link_definitions[$reverse_provider . '/' . $link_type . '/reverse'] = array(
                                    'provider' => $reverse_provider,
                                    'description' => $reverse_description,
                                    'type' => $link_type,
                                    'readonly' => (isset($link_definition['reverse']['readonly']) && $link_definition['reverse']['readonly']),
                                );
                            }
                        }
                    }
                }
            }
        }
    }

    /*
     * This function returns the list of link definitions if the 
     * driver == 'none' this should be empty.
     * 
     * @access public
     *
     * @return array The list of link definitions.
     */
    function listLinkTypes()
    {
        return $this->_link_definitions;
    }

    /*
     * This function returns a list of links for the specified link_type 
     * and the specified from_parameters
     *
     * @param link_type
     *
     * @param from_parameters
     *
     * @access public
     * 
     * @return array The list of links
     */
    function listLinks($link_type, $from_parameters) 
    {
        global $registry;
        $this->_getLinks($from_parameters);
        return $this->_links[serialize($from_parameters)][$link_type];
    }
    
    /*
     * This function provides the general (shared) user interface for links.
     * TODO: This probably needs work.
     *
     * @param hash $link_parameters  
     *
     * @access public
     * 
     * @return array The list of links
     */
    function viewLinks($link_parameters) 
    {
        global $registry;

        foreach ($this->_link_definitions as $link_type => $link_definition) {
            $app = $this->_link_definitions[$link_type]['provider'];
            $type = $this->_link_definitions[$link_type]['type'];
            include $registry->getParam('templates', 'horde') . '/links/header.inc';
            include $registry->getParam('templates', 'horde') . '/links/links.inc';
            include $registry->getParam('templates', 'horde') . '/links/footer.inc';
        }
    }
    /*
     * 
     * This function provides (part of) the general (shared) user interface 
     * for links.
     * TODO: This definitly needs work. E.g. a delete link is generated here, 
     * while the function names signifies that only the description is printed.
     * This is an initial quick and dirty implementation.
     *
     * @param string $link_type
     *
     * @param hash $from_parameters  
     *
     * @param hash $to_parameters  
     *
     * @access public (TODO: Is this really true? Do we need this?)
     * 
     */
    function printDescription($link_type, $from_parameters, $to_parameters) 
    {
        global $registry;

        $app = $this->_link_definitions[$link_type]['provider'];

        if ($registry->hasMethod($app . '/' . 'getLinkDescription')) {
            $description = $registry->call($app . '/' . 'getLinkDescription', $to_parameters); 
            if (is_a($description, 'PEAR_Error')) {
                echo "<pre style='color:red'>\n";
                print_r($description);
                echo "</pre>\n";
            } else {
                $url = $registry->link($app . '/followLink', $to_parameters);
                if (!is_a($url, 'PEAR_Error')) { 
                    $description  = '<a href="' . Horde::url($url) . '">' . $description . '</a>';
                }
                echo $description;
                if ($registry->hasMethod('deleteLink','horde') && !$this->_link_definitions[$link_type]['readonly']) {
                    $url = $registry->linkByPackage('horde','deleteLink',
                        array(
                            'from_application' => $registry->getParam('provides'),
                            'from_parameters' => serialize($from_parameters),
                            'to_application' => $app,
                            'to_parameters' => serialize($to_parameters),
                            'link_type' => $this->_link_definitions[$link_type]['type'],
                            'url' => Horde::selfURL(true,false)
                        )
                    );
                    echo '&nbsp;<a href="'. Horde::url($url) .'" class="button" >Delete</a>';
                }
            }
        } else {
            print "no description:<br>";
            print_r($to_parameters);
        }
    }
}

?>
